@@ -1,4 +1,4 @@
-// Rcpp.cpp: Part of the R/C++ interface class library, Version 4.0
+// Rcpp.cpp: Part of the R/C++ interface class library, Version 5.0
 //
 // Copyright (C) 2005-2006 Dominick Samperi
 //
@@ -23,8 +23,13 @@ RcppParams::RcppParams(SEXP params) {
 	throw std::range_error("RcppParams: non-list passed to constructor");
     int len = length(params);
     SEXP names = getAttrib(params, R_NamesSymbol);
+    if(names == R_NilValue)
+	throw std::range_error("RcppParams: list must have named elements");
     for(int i = 0; i < len; i++) {
-	pmap[string(CHAR(STRING_ELT(names,i)))] = i;
+	string nm = string(CHAR(STRING_ELT(names,i)));
+	if(nm.size() == 0)
+	    throw std::range_error("RcppParams: all list elements must be named");
+	pmap[nm] = i;
     }
     _params = params;
 }
@@ -42,46 +47,68 @@ void RcppParams::checkNames(char *inputNames[], int len) {
 RcppFrame::RcppFrame(SEXP df) {
     if(!isNewList(df))
 	throw std::range_error("RcppFrame::RcppFrame: invalid data frame.");
-    int ncols = length(df);
+    int ncol = length(df);
     SEXP names = getAttrib(df, R_NamesSymbol);
-    colNames_.resize(ncols);
+    colNames.resize(ncol);
     SEXP colData = VECTOR_ELT(df,0); // First column of data.
-    int nrows = length(colData);
-    if(nrows == 0)
+    int nrow = length(colData);
+    if(nrow == 0)
 	throw std::range_error("RcppFrame::RcppFrame: zero lenth column.");
 
     // Allocate storage for table.
-    table.resize(nrows);
-    for(int r = 0; r < nrows; r++)
-	table[r].resize(ncols);
+    table.resize(nrow);
+    for(int r = 0; r < nrow; r++)
+	table[r].resize(ncol);
     
-    for(int i=0; i < ncols; i++) {
-	colNames_[i] = string(CHAR(STRING_ELT(names,i)));
+    for(int i=0; i < ncol; i++) {
+	colNames[i] = string(CHAR(STRING_ELT(names,i)));
 	SEXP colData = VECTOR_ELT(df,i);
-	if(!isVector(colData) || length(colData) != nrows)
+	if(!isVector(colData) || length(colData) != nrow)
 	    throw std::range_error("RcppFrame::RcppFrame: invalid column.");
+
+	// Check for Date class. Currently R stores the date ordinal in a
+	// real value. We check for Date under both Real and Integer values
+	// as insurance against future changes.
+	bool isDateClass = false;
+	SEXP classname = getAttrib(colData, R_ClassSymbol);
+	if(classname != R_NilValue)
+	    isDateClass = (strcmp(CHAR(STRING_ELT(classname,0)),"Date") == 0);
+
 	if(isReal(colData)) {
-	    for(int j=0; j < nrows; j++)
-		table[j][i].setDoubleValue(REAL(colData)[j]);
+	    if(isDateClass) {
+		for(int j=0; j < nrow; j++) // Column of Date's
+		    table[j][i].setDateValue(RcppDate((int)REAL(colData)[j]));
+	    }
+	    else // Column of REAL's
+		for(int j=0; j < nrow; j++)
+		    table[j][i].setDoubleValue(REAL(colData)[j]);
 	}
 	else if(isInteger(colData)) {
-	    for(int j=0; j < nrows; j++)
-		table[j][i].setIntValue(INTEGER(colData)[j]);
+	    if(isDateClass) {
+		for(int j=0; j < nrow; j++) // Column of Date's
+		    table[j][i].setDateValue(RcppDate(INTEGER(colData)[j]));
+	    }
+	    else
+		for(int j=0; j < nrow; j++)
+		    table[j][i].setIntValue(INTEGER(colData)[j]);
 	}
 	else if(isString(colData)) { // Non-factor string column
-	    for(int j=0; j < nrows; j++)
+	    for(int j=0; j < nrow; j++)
 		table[j][i].setStringValue(string(CHAR(STRING_ELT(colData,j))));
 	}
 	else if (isFactor(colData)) { // Factor column.
 	    SEXP names = getAttrib(colData, R_LevelsSymbol);
-	    for(int j=0; j < nrows; j++) {
-		int level = INTEGER(colData)[j];
-		table[j][i].setFactorValue(level,
-			    string(CHAR(STRING_ELT(names,level-1))));
-	    }
+	    int numLevels = length(names);
+	    string *levelNames = new string[numLevels];
+	    for(int k=0; k < numLevels; k++)
+		levelNames[k] = string(CHAR(STRING_ELT(names,k)));
+	    for(int j=0; j < nrow; j++)
+		table[j][i].setFactorValue(levelNames, numLevels,
+					   INTEGER(colData)[j]);
+	    delete [] levelNames;
 	}
 	else if(isLogical(colData)) {
-	    for(int j=0; j < nrows; j++) {
+	    for(int j=0; j < nrow; j++) {
 		table[j][i].setLogicalValue(INTEGER(colData)[j]);
 	    }
 	}
@@ -170,38 +197,56 @@ string RcppParams::getStringValue(string name) {
     return ""; // never get here
 }
 
-#ifdef USING_QUANTLIB
-Date RcppParams::getDateValue(string name) {
+RcppDate RcppParams::getDateValue(string name) {
     map<string,int>::iterator iter = pmap.find(name);
     if(iter == pmap.end()) {
 	string mesg = "getDateValue: no such name: ";
 	throw std::range_error(mesg+name);
     }
     int posn = iter->second;
-    int day=0, month=0, year=0;
-    SEXP dateSEXP = VECTOR_ELT(_params, posn);
-    if(!isNumeric(dateSEXP) || length(dateSEXP) != 3) {
-	string mesg = "getDateValue: invalid date: ";
+    SEXP elt = VECTOR_ELT(_params,posn);
+    if(!isNumeric(elt) || length(elt) != 1) {
+	string mesg = "getDateValue: invalide date: ";
 	throw std::range_error(mesg+name);
     }
-    if(isInteger(dateSEXP)) {
-	month   = INTEGER(dateSEXP)[0];
-	day   = INTEGER(dateSEXP)[1];
-	year  = INTEGER(dateSEXP)[2];
-    }
-    else if(isReal(dateSEXP)) {
-	month   = (int)REAL(dateSEXP)[0];
-	day   = (int)REAL(dateSEXP)[1];
-	year  = (int)REAL(dateSEXP)[2];
-    }
+
+    int d;
+    if(isReal(elt)) // R stores julian value in a double.
+	d = (int)REAL(elt)[0];
     else {
 	string mesg = "getDateValue: invalid value for: ";
 	throw std::range_error(mesg+name);
     }
-    Date d(day, (Month)month, year);
-    return d;
+    return RcppDate(d);
+}
+
+RcppDateVector::RcppDateVector(SEXP vec) {
+    int i;
+    if(!isNumeric(vec) || isMatrix(vec) || isLogical(vec))
+	throw std::range_error("RcppVector: invalid numeric vector in constructor");
+    int len = length(vec);
+    if(len == 0)
+	throw std::range_error("RcppVector: null vector in constructor");
+    v = new RcppDate[len];
+    for(i = 0; i < len; i++)
+	v[i] = RcppDate((int)REAL(vec)[i]);
+    length = len;
+}
+
+RcppStringVector::RcppStringVector(SEXP vec) {
+    int i;
+    if(isMatrix(vec) || isLogical(vec))
+	throw std::range_error("RcppVector: invalid numeric vector in constructor");
+    if(!isString(vec))
+	throw std::range_error("RcppStringVector: invalid string");
+    int len = length(vec);
+    if(len == 0)
+	throw std::range_error("RcppVector: null vector in constructor");
+    v = new string[len];
+    for(i = 0; i < len; i++)
+	v[i] = string(CHAR(STRING_ELT(vec,i)));
+    length = len;
 }
-#endif
 
 template <typename T>
 RcppVector<T>::RcppVector(SEXP vec) {
@@ -215,17 +260,13 @@ RcppVector<T>::RcppVector(SEXP vec) {
     // isMatrix (in which case we don't have a vector).
     if(!isNumeric(vec) || isMatrix(vec) || isLogical(vec))
 	throw std::range_error("RcppVector: invalid numeric vector in constructor");
-
     len = length(vec);
-    if(len == 0)
-	throw std::range_error("RcppVector: null vector in constructor");
-    int isInt = isInteger(vec);
     v = (T *)R_alloc(len, sizeof(T));
-    if(isInt) {
+    if(isInteger(vec)) {
 	for(i = 0; i < len; i++)
 	    v[i] = (T)(INTEGER(vec)[i]);
     }	
-    else {
+    else if(isReal(vec)) {
 	for(i = 0; i < len; i++)
 	    v[i] = (T)(REAL(vec)[i]);
     }
@@ -332,6 +373,17 @@ template class RcppVector<double>;
 template class RcppMatrix<int>;
 template class RcppMatrix<double>;
 
+void RcppResultSet::add(string name, RcppDate& date) {
+    SEXP value = PROTECT(allocVector(REALSXP, 1));
+    numProtected++;
+    REAL(value)[0] = date.getJDN() - RcppDate::Jan1970Offset;
+    SEXP dateclass = PROTECT(allocVector(STRSXP,1));
+    numProtected++;
+    SET_STRING_ELT(dateclass, 0, mkChar("Date"));
+    setAttrib(value, R_ClassSymbol, dateclass); 
+    values.push_back(make_pair(name, value));
+}
+
 void RcppResultSet::add(string name, double x) {
     SEXP value = PROTECT(allocVector(REALSXP, 1));
     numProtected++;
@@ -354,6 +406,8 @@ void RcppResultSet::add(string name, string strvalue) {
 }
 
 void RcppResultSet::add(string name, double *vec, int len) {
+    if(vec == 0)
+	throw std::range_error("RcppResultSet::add: NULL double vector");
     SEXP value = PROTECT(allocVector(REALSXP, len));
     numProtected++;
     for(int i = 0; i < len; i++)
@@ -361,7 +415,31 @@ void RcppResultSet::add(string name, double *vec, int len) {
     values.push_back(make_pair(name, value));
 }
 
+void RcppResultSet::add(string name, RcppDateVector& datevec) {
+    SEXP value = PROTECT(allocVector(REALSXP, datevec.size()));
+    numProtected++;
+    for(int i = 0; i < datevec.size(); i++) {
+	REAL(value)[i] = datevec(i).getJDN() - RcppDate::Jan1970Offset;
+    }
+    SEXP dateclass = PROTECT(allocVector(STRSXP,1));
+    numProtected++;
+    SET_STRING_ELT(dateclass, 0, mkChar("Date"));
+    setAttrib(value, R_ClassSymbol, dateclass); 
+    values.push_back(make_pair(name, value));
+}
+
+void RcppResultSet::add(string name, RcppStringVector& stringvec) {
+    int len = (int)stringvec.size();
+    SEXP value = PROTECT(allocVector(STRSXP, len));
+    numProtected++;
+    for(int i = 0; i < len; i++)
+        SET_STRING_ELT(value, i, mkChar(stringvec(i).c_str()));
+    values.push_back(make_pair(name, value));
+}
+
 void RcppResultSet::add(string name, int *vec, int len) {
+    if(vec == 0)
+	throw std::range_error("RcppResultSet::add: NULL int vector");
     SEXP value = PROTECT(allocVector(INTSXP, len));
     numProtected++;
     for(int i = 0; i < len; i++)
@@ -370,6 +448,8 @@ void RcppResultSet::add(string name, int *vec, int len) {
 }
 
 void RcppResultSet::add(string name, double **mat, int nx, int ny) {
+    if(mat == 0)
+	throw std::range_error("RcppResultSet::add: NULL double matrix");
     SEXP value = PROTECT(allocMatrix(REALSXP, nx, ny));
     numProtected++;
     for(int i = 0; i < nx; i++)
@@ -379,6 +459,8 @@ void RcppResultSet::add(string name, double **mat, int nx, int ny) {
 }
 
 void RcppResultSet::add(string name, int **mat, int nx, int ny) {
+    if(mat == 0)
+	throw std::range_error("RcppResultSet::add: NULL int matrix");
     SEXP value = PROTECT(allocMatrix(INTSXP, nx, ny));
     numProtected++;
     for(int i = 0; i < nx; i++)
@@ -388,6 +470,8 @@ void RcppResultSet::add(string name, int **mat, int nx, int ny) {
 }
 
 void RcppResultSet::add(string name, vector<string>& vec) {
+    if(vec.size() == 0)
+	throw std::range_error("RcppResultSet::add; zero length vector<string>");
     int len = (int)vec.size();
     SEXP value = PROTECT(allocVector(STRSXP, len));
     numProtected++;
@@ -397,6 +481,8 @@ void RcppResultSet::add(string name, vector<string>& vec) {
 }
 
 void RcppResultSet::add(string name, vector<int>& vec) {
+    if(vec.size() == 0)
+	throw std::range_error("RcppResultSet::add; zero length vector<int>");
     int len = (int)vec.size();
     SEXP value = PROTECT(allocVector(INTSXP, len));
     numProtected++;
@@ -406,6 +492,8 @@ void RcppResultSet::add(string name, vector<int>& vec) {
 }
 
 void RcppResultSet::add(string name, vector<double>& vec) {
+    if(vec.size() == 0)
+	throw std::range_error("RcppResultSet::add; zero length vector<double>");
     int len = (int)vec.size();
     SEXP value = PROTECT(allocVector(REALSXP, len));
     numProtected++;
@@ -415,6 +503,10 @@ void RcppResultSet::add(string name, vector<double>& vec) {
 }
 
 void RcppResultSet::add(string name, vector<vector<int> >& mat) {
+    if(mat.size() == 0)
+	throw std::range_error("RcppResultSet::add: zero length vector<vector<int> >");
+    else if(mat[0].size() == 0)
+	throw std::range_error("RcppResultSet::add: no columns in vector<vector<int> >");
     int nx = (int)mat.size();
     int ny = (int)mat[0].size();
     SEXP value = PROTECT(allocMatrix(INTSXP, nx, ny));
@@ -426,6 +518,10 @@ void RcppResultSet::add(string name, vector<vector<int> >& mat) {
 }
 
 void RcppResultSet::add(string name, vector<vector<double> >& mat) {
+    if(mat.size() == 0)
+	throw std::range_error("RcppResultSet::add: zero length vector<vector<double> >");
+    else if(mat[0].size() == 0)
+	throw std::range_error("RcppResultSet::add: no columns in vector<vector<double> >");
     int nx = (int)mat.size();
     int ny = (int)mat[0].size();
     SEXP value = PROTECT(allocMatrix(REALSXP, nx, ny));
@@ -437,7 +533,7 @@ void RcppResultSet::add(string name, vector<vector<double> >& mat) {
 }
 
 void RcppResultSet::add(string name, RcppVector<int>& vec) {
-    int len = vec.getLength();
+    int len = vec.size();
     int *a = vec.cVector();
     SEXP value = PROTECT(allocVector(INTSXP, len));
     numProtected++;
@@ -447,7 +543,7 @@ void RcppResultSet::add(string name, RcppVector<int>& vec) {
 }
 
 void RcppResultSet::add(string name, RcppVector<double>& vec) {
-    int len = vec.getLength();
+    int len = vec.size();
     double *a = vec.cVector();
     SEXP value = PROTECT(allocVector(REALSXP, len));
     numProtected++;
@@ -483,48 +579,69 @@ void RcppResultSet::add(string name, RcppMatrix<double>& mat) {
 void RcppResultSet::add(string name, RcppFrame& frame) {
     vector<string> colNames = frame.getColNames();
     vector<vector<ColDatum> > table = frame.getTableData();
-    int ncols = colNames.size();
-    int nrows = table.size();
-    SEXP rl = PROTECT(allocVector(VECSXP,ncols));
-    SEXP nm = PROTECT(allocVector(STRSXP,ncols));
+    int ncol = colNames.size();
+    int nrow = table.size();
+    SEXP rl = PROTECT(allocVector(VECSXP,ncol));
+    SEXP nm = PROTECT(allocVector(STRSXP,ncol));
     numProtected += 2;
-    for(int i=0; i < ncols; i++) {
-	SEXP value;
+    for(int i=0; i < ncol; i++) {
+	SEXP value, names;
 	if(table[0][i].getType() == COLTYPE_DOUBLE) {
-	    value = PROTECT(allocVector(REALSXP,nrows));
+	    value = PROTECT(allocVector(REALSXP,nrow));
 	    numProtected++;
-	    for(int j=0; j < nrows; j++)
+	    for(int j=0; j < nrow; j++)
 		REAL(value)[j] = table[j][i].getDoubleValue();
 	}
 	else if(table[0][i].getType() == COLTYPE_INT) {
-	    value = PROTECT(allocVector(INTSXP,nrows));
+	    value = PROTECT(allocVector(INTSXP,nrow));
 	    numProtected++;
-	    for(int j=0; j < nrows; j++)
+	    for(int j=0; j < nrow; j++)
 		INTEGER(value)[j] = table[j][i].getIntValue();
 	}
 	else if(table[0][i].getType() == COLTYPE_FACTOR) {
-	    value = PROTECT(allocVector(STRSXP,nrows));
+	    value = PROTECT(allocVector(INTSXP,nrow));
 	    numProtected++;
-	    for(int j=0; j < nrows; j++) { // Ignore level for now.
-		SET_STRING_ELT(value, j, mkChar(table[j][i].getFactorName().c_str()));
+	    int levels = table[0][i].getFactorNumLevels();
+	    names = PROTECT(allocVector(STRSXP,levels));
+	    numProtected++;
+	    string *levelNames = table[0][i].getFactorLevelNames();
+	    for(int k=0; k < levels; k++)
+		SET_STRING_ELT(names, k, mkChar(levelNames[k].c_str()));
+	    for(int j=0; j < nrow; j++) {
+		int level = table[j][i].getFactorLevel();
+		INTEGER(value)[j] = level;
 	    }
-		
+	    setAttrib(value, R_LevelsSymbol, names);
+	    SEXP factorclass = PROTECT(allocVector(STRSXP,1));
+	    numProtected++;
+	    SET_STRING_ELT(factorclass, 0, mkChar("factor"));
+	    setAttrib(value, R_ClassSymbol, factorclass); 
 	}
 	else if(table[0][i].getType() == COLTYPE_STRING) {
-	    value = PROTECT(allocVector(STRSXP,nrows));
+	    value = PROTECT(allocVector(STRSXP,nrow));
 	    numProtected++;
-	    for(int j=0; j < nrows; j++) {
+	    for(int j=0; j < nrow; j++) {
 		SET_STRING_ELT(value, j, mkChar(table[j][i].getStringValue().c_str()));
 	    }
 		
 	}
 	else if(table[0][i].getType() == COLTYPE_LOGICAL) {
-	    value = PROTECT(allocVector(LGLSXP,nrows));
+	    value = PROTECT(allocVector(LGLSXP,nrow));
 	    numProtected++;
-	    for(int j=0; j < nrows; j++) {
+	    for(int j=0; j < nrow; j++) {
 		LOGICAL(value)[j] = table[j][i].getLogicalValue();
 	    }
 	}
+	else if(table[0][i].getType() == COLTYPE_DATE) {
+	    value = PROTECT(allocVector(REALSXP,nrow));
+	    numProtected++;
+	    for(int j=0; j < nrow; j++)
+		REAL(value)[j] = table[j][i].getDateRCode();
+	    SEXP dateclass = PROTECT(allocVector(STRSXP,1));
+	    numProtected++;
+	    SET_STRING_ELT(dateclass, 0, mkChar("Date"));
+	    setAttrib(value, R_ClassSymbol, dateclass); 
+	}
 	else {
 	    throw std::range_error("RcppResultSet::add invalid column type");
 	}
@@ -556,22 +673,212 @@ SEXP RcppResultSet::getReturnList() {
 }
 
 #ifdef USING_QUANTLIB
+
+// Conversion from QuantLib Date to RcppDate.
+RcppDate::RcppDate(Date dateQL) {
+    day = (int)dateQL.dayOfMonth();
+    month = (int)dateQL.month();
+    year  = (int)dateQL.year();
+    mdy2jdn();
+}
+
+// Conversion from RcppDate to QuantLib Date.
+RcppDate::operator Date() const {
+    Date d(day, (Month)month, year);
+    return d;
+}
+
+// Print a QuantLib Date.
 ostringstream& operator<<(ostringstream& os, const Date& d) {
     os << d.month() << " " << d.weekday() << ", " << d.year();
     return os;
 }
+
 #endif
 
+// Print an RcppDate.
+ostream& operator<<(ostream& os, const RcppDate& date) {
+    os << date.getYear() << "-" << date.getMonth() << "-" << date.getDay();
+    return os;
+}
+
+#ifdef RCPP_DATE_OPS
+
+// A few basic date operations.
+RcppDate operator+(const RcppDate& date, int offset) {
+    RcppDate temp(date.month, date.day, date.year);
+    temp.jdn += offset;
+    temp.jdn2mdy();
+    return temp;
+}
+
+int operator-(const RcppDate& date2, const RcppDate& date1) {
+    return date2.jdn - date1.jdn;
+}
+
+bool  operator<(const RcppDate &date1, const RcppDate& date2) {
+    return date1.jdn < date2.jdn;
+}
+
+bool  operator>(const RcppDate &date1, const RcppDate& date2) {
+    return date1.jdn > date2.jdn;
+}
+
+bool  operator>=(const RcppDate &date1, const RcppDate& date2) {
+    return date1.jdn >= date2.jdn;
+}
+
+bool  operator<=(const RcppDate &date1, const RcppDate& date2) {
+    return date1.jdn <= date2.jdn;
+}
+
+bool  operator==(const RcppDate &date1, const RcppDate& date2) {
+    return date1.jdn == date2.jdn;
+}
+
+#endif
+
+// Offset used to convert from R date representation to Julian day number.
+const int RcppDate::Jan1970Offset = 2440588;
+
+// The Julian day number (jdn) is the number of days since Monday,
+// Jan 1, 4713BC (year = -4712). Here 1BC is year 0, 2BC is year -1, etc.
+// On the other hand, R measures days since Jan 1, 1970, and these dates are
+// converted to jdn's by adding Jan1970Offset.
+//
+// mdy2jdn and jdn2mdy are inverse functions for dates back to 
+// year = -4799 (4800BC).
+//
+// See the Wikipedia entry on Julian day number for more information 
+// on these algorithms.
+//
+
+// Transform month/day/year to Julian day number.
+void RcppDate::mdy2jdn() {
+    int m = month, d = day, y = year;
+    int a = (14 - m)/12;
+    y += 4800 - a;
+    m += 12*a - 3;
+    jdn = (d + (153*m + 2)/5 + 365*y
+	   + y/4 - y/100 + y/400 - 32045);
+}
+
+// Transform from Julian day number to month/day/year.
+void RcppDate::jdn2mdy() {
+    int jul = jdn + 32044;
+    int g = jul/146097;
+    int dg = jul % 146097;
+    int c = (dg/36524 + 1)*3/4;
+    int dc = dg - c*36524;
+    int b = dc/1461;
+    int db = dc % 1461;
+    int a = (db/365 + 1)*3/4;
+    int da = db - a*365;
+    int y = g*400 + c*100 + b*4 + a;
+    int m = (da*5 + 308)/153 - 2;
+    int d = da - (m + 4)*153 /5 + 122;
+    y = y - 4800 + (m + 2)/12;
+    m = (m + 2) % 12 + 1;
+    d = d + 1;
+    month = m;
+    day   = d;
+    year  = y;
+}
+
+SEXP RcppFunction::listCall() {
+    if(names.size() != (unsigned)listSize)
+	throw std::range_error("listCall: no. of names != no. of items");
+    if(currListPosn != listSize)
+	throw std::range_error("listCall: list has incorrect size");
+    SEXP nm = PROTECT(allocVector(STRSXP,listSize));
+    numProtected++;
+    for(int i=0; i < listSize; i++)
+	SET_STRING_ELT(nm, i, mkChar(names[i].c_str()));
+    setAttrib(listArg, R_NamesSymbol, nm);
+    SEXP R_fcall;
+    PROTECT(R_fcall = lang2(fn, R_NilValue));
+    numProtected++;
+    SETCADR(R_fcall, listArg);
+    SEXP result = eval(R_fcall, R_NilValue);
+    names.clear();
+    listSize = currListPosn = 0; // Ready for next call.
+    return result;
+}
+
+SEXP RcppFunction::vectorCall() {
+    if(vectorArg == R_NilValue)
+	throw std::range_error("vectorCall: vector has not been set");
+    SEXP R_fcall;
+    PROTECT(R_fcall = lang2(fn, R_NilValue));
+    numProtected++;
+    SETCADR(R_fcall, vectorArg);
+    SEXP result = eval(R_fcall, R_NilValue);
+    vectorArg = R_NilValue; // Ready for next call.
+    return result;
+}
+
+void RcppFunction::setRVector(vector<double>& v) {
+    vectorArg = PROTECT(allocVector(REALSXP,v.size()));
+    numProtected++;
+    for(int i=0; i < (int)v.size(); i++)
+	REAL(vectorArg)[i] = v[i];
+}
+
+void RcppFunction::setRListSize(int n) {
+    listSize = n;
+    listArg = PROTECT(allocVector(VECSXP, n));
+    numProtected++;
+}
+
+void RcppFunction::appendToRList(string name, double value) {
+    if(currListPosn < 0 || currListPosn >= listSize)
+	throw std::range_error("appendToRList(double): list posn out of range");
+    SEXP valsxp = PROTECT(allocVector(REALSXP,1));
+    numProtected++;
+    REAL(valsxp)[0] = value;
+    SET_VECTOR_ELT(listArg, currListPosn++, valsxp);
+    names.push_back(name);
+}
+
+void RcppFunction::appendToRList(string name, int value) {
+    if(currListPosn < 0 || currListPosn >= listSize)
+	throw std::range_error("appendToRlist(int): posn out of range");
+    SEXP valsxp = PROTECT(allocVector(INTSXP,1));
+    numProtected++;
+    INTEGER(valsxp)[0] = value;
+    SET_VECTOR_ELT(listArg, currListPosn++, valsxp);
+    names.push_back(name);
+}
+
+void RcppFunction::appendToRList(string name, string value) {
+    if(currListPosn < 0 || currListPosn >= listSize)
+	throw std::range_error("appendToRlist(string): posn out of range");
+    SEXP valsxp = PROTECT(allocVector(STRSXP,1));
+    numProtected++;
+    SET_STRING_ELT(valsxp, 0, mkChar(value.c_str()));
+    SET_VECTOR_ELT(listArg, currListPosn++, valsxp);
+    names.push_back(name);
+}
+
+void RcppFunction::appendToRList(string name, RcppDate& date) {
+    if(currListPosn < 0 || currListPosn >= listSize)
+	throw std::range_error("appendToRlist(RcppDate): list posn out of range");
+    SEXP valsxp = PROTECT(allocVector(REALSXP,1));
+    numProtected++;
+    REAL(valsxp)[0] = date.getJDN() - RcppDate::Jan1970Offset;
+    SEXP dateclass = PROTECT(allocVector(STRSXP, 1));
+    numProtected++;
+    SET_STRING_ELT(dateclass, 0, mkChar("Date"));
+    setAttrib(valsxp, R_ClassSymbol, dateclass);
+    SET_VECTOR_ELT(listArg, currListPosn++, valsxp);
+    names.push_back(name);
+}
+
 #include <string.h>
 
-// This function copies the message string to R-managed memory so the
-// original C++ message object can be destroyed (when it goes out of
-// scope before returning to R).
-//
-// Thanks to Paul Roebuck for pointing out that the exception
-// object's memory will not be reclaimed if error() is called inside of
-// a catch block (due to a setjmp() call), and for suggesting the
-// work-around.
+// Paul Roebuck has observed that the memory used by an exception message
+// is not reclaimed if error() is called inside of a catch block (due to
+// a setjmp() call), and he suggested the following work-around.
 char *copyMessageToR(const char* const mesg) {
     char* Rmesg;
     char* prefix = "Exception: ";
